// Normal N() and inverse normal cdf Ni()

// N(0,1) density
double f(double x) {
	double pi = 3.14159265358979323846264338;   // pi
	return exp(-x*x*0.5)/sqrt(2*pi);
}

// Boole's Rule
double Boole(double StartPoint, double EndPoint, int n) {
vector<double> X(n+1, 0.0);
vector<double> Y(n+1, 0.0);
double delta_x = (EndPoint - StartPoint)/double(n);
for (int i=0; i<=n; i++) {
	X[i] = StartPoint + i*delta_x;
	Y[i] = f(X[i]);
}
double sum = 0;
for (int t=0; t<=(n-1)/4; t++) {
	int ind = 4*t;
    sum += (1/45.0)*(14*Y[ind] + 64*Y[ind+1] + 24*Y[ind+2] + 64*Y[ind+3] + 14*Y[ind+4])*delta_x;
}
return sum;
}

// N(0,1) cdf by Boole's Rule
double N(double x) {
	return Boole(-10.0, x, 240);
}

// Bisection Algorithm
double Bisection(double prob, double a, double b)
{
const int MaxIter = 500;
double Tol = 0.00001;
double midP, midCdif;
double  lowCdif = prob - N(a);
double highCdif = prob - N(b);
if (lowCdif*highCdif > 0)
{
	double Temp = lowCdif;
	lowCdif = highCdif;
	highCdif = Temp;
}
else
for (int i=0; i<=MaxIter; i++)
{
    midP = (a + b) / 2.0;
	midCdif = prob - N(midP);
	if (abs(midCdif)<Tol) goto LastLine;
	else
	{
        if (midCdif>0) a = midP;
		else b = midP;
	}
}
LastLine:
return midP;
}

// Inverse cumulative N(0,1) by bisection algorithm
double Ni(double prob) {
	return Bisection(prob, -10.0, 10.0);
}
